from World import world, string, character
from generator import StringGenerator
import constants
import pygame


class TestStringGenerator(object):
    
    def setup_method(self, method):
        self.world = world.World()
        screen = pygame.Surface((100, 100))
        class fake_engine(object):
            def get_screen(self):
                return screen
        self.generator = StringGenerator(fake_engine(), self.world)
    
    def test_should_generate_string(self):
        rope = self.generator.generate()
        assert type(rope) == string.String

    # is that really necessary?
    #def test_generated_strings_should_be_above_screen(self):
    #   self.assert_many_times(lambda str: str.position()[1]+str.length() <= 0, True)
        
    def test_generated_strings_should_be_within_limits(self):
        self.assert_many_times(lambda str: str.position()[0] >= 0 and str.position()[1] <= constants.STRING_FLOOR, True)
    
    def test_generated_string_length_should_be_within_limits(self):
        self.assert_many_times(lambda str: str.length() <= constants.MAX_STRING_LENGTH and str.length() >= constants.MIN_STRING_LENGTH, True)
    
    def test_should_populate_strings_if_there_are_none_in_the_world(self):
        self.assert_string_count(0)
        self.generator.populate_world()
        assert len(self.world.all_strings()) > 0
    
    def test_should_not_populate_world_with_strings_if_there_are_enough_already(self):
        for i in xrange(constants.MAX_WORLD_STRINGS):
            self.world.add_object(string.String())
        self.assert_string_count(constants.MAX_WORLD_STRINGS)
        self.generator.populate_world()
        self.assert_string_count(constants.MAX_WORLD_STRINGS)
                                                           
    def assert_string_count(self, count):
        assert len(self.world.all_strings()) == count
    
    def assert_many_times(self, assertion, result):
        for i in xrange(50):
            str = self.generator.generate()
            if not assertion(str) == result:
                raise AssertionError("Expected %s but got %s for %s" % (assertion(str), result, str))
